@@ -44,11 +44,19 @@ class BinaryReaderNop : public BinaryReaderDelegate {
   /* Type section */
   Result BeginTypeSection(Offset size) override { return Result::Ok; }
   Result OnTypeCount(Index count) override { return Result::Ok; }
-  Result OnType(Index index,
-                Index param_count,
-                Type* param_types,
-                Index result_count,
-                Type* result_types) override {
+  Result OnFuncType(Index index,
+                    Index param_count,
+                    Type* param_types,
+                    Index result_count,
+                    Type* result_types) override {
+    return Result::Ok;
+  }
+  Result OnStructType(Index index,
+                      Index field_count,
+                      TypeMut* fields) override {
+    return Result::Ok;
+  }
+  Result OnArrayType(Index index, TypeMut field) override {
     return Result::Ok;
   }
   Result EndTypeSection() override { return Result::Ok; }
@@ -205,6 +213,9 @@ class BinaryReaderNop : public BinaryReaderDelegate {
   Result OnAtomicWaitExpr(Opcode, uint32_t, Address) override {
     return Result::Ok;
   }
+  Result OnAtomicFenceExpr(uint32_t) override {
+    return Result::Ok;
+  }
   Result OnAtomicNotifyExpr(Opcode, uint32_t, Address) override {
     return Result::Ok;
   }
@@ -302,12 +313,14 @@ class BinaryReaderNop : public BinaryReaderDelegate {
   Result OnElemSegmentCount(Index count) override { return Result::Ok; }
   Result BeginElemSegment(Index index,
                           Index table_index,
-                          uint8_t flags,
-                          Type elem_type) override {
+                          uint8_t flags) override {
     return Result::Ok;
   }
   Result BeginElemSegmentInitExpr(Index index) override { return Result::Ok; }
   Result EndElemSegmentInitExpr(Index index) override { return Result::Ok; }
+  Result OnElemSegmentElemType(Index index, Type elem_type) override {
+    return Result::Ok;
+  }
   Result OnElemSegmentElemExprCount(Index index, Index count) override {
     return Result::Ok;
   }
