@@ -926,7 +926,8 @@ Result WastParser::ParseNat(uint64_t* out_nat) {
 
   Token token = Consume();
   string_view sv = token.literal().text;
-  if (Failed(ParseUint64(sv.begin(), sv.end(), out_nat))) {
+  if (Failed(ParseUint64(sv.begin(), sv.end(), out_nat)) ||
+      *out_nat > 0xffffffffu) {
     Error(token.loc, "invalid int \"" PRIstringview "\"",
           WABT_PRINTF_STRING_VIEW_ARG(sv));
   }
@@ -2651,8 +2652,11 @@ Result WastParser::ParseExpr(ExprList* exprs) {
         auto expr = MakeUnique<TryExpr>(loc);
         CHECK_RESULT(ParseLabelOpt(&expr->block.label));
         CHECK_RESULT(ParseBlockDeclaration(&expr->block.decl));
+        EXPECT(Lpar);
+        EXPECT(Do);
         CHECK_RESULT(ParseInstrList(&expr->block.exprs));
         expr->block.end_loc = GetLocation();
+        EXPECT(Rpar);
         EXPECT(Lpar);
         EXPECT(Catch);
         CHECK_RESULT(ParseTerminatingInstrList(&expr->catch_));
